#include "config.hpp"
#include "init_user_directory.hpp"
#include "server_service.hpp"
#include "server_env.hpp"
#include "templates.hpp"
#include "config.hpp"
#include <boost/filesystem.hpp>
#include <iostream>
#include <fstream>
#include <sstream>
#include <cstdlib>
#include <chrono>
#include <iomanip>

namespace fs = boost::filesystem;

namespace dropshell {

std::vector<std::string> get_server_services(const std::string& server_name) {
    std::vector<std::string> services;
    std::string user_dir;
    
    if (!get_user_directory(user_dir)) {
        std::cerr << "Error: User directory not set" << std::endl;
        return services;
    }
    
    fs::path server_dir = fs::path(user_dir) / "servers" / server_name;
    if (!fs::exists(server_dir)) {
        std::cerr << "Error: Server directory not found" << std::endl;
        return services;
    }
    
    // Look for .env files in the server directory
    for (const auto& entry : fs::directory_iterator(server_dir)) {
        if (entry.path().extension() == ".env" && entry.path().filename().string() != "_server.env") {
            services.push_back(entry.path().stem().string());
        }
    }
    
    return services;
}

class server_service {
private:
    std::string m_server_name;
    std::string m_service_name;
    std::unique_ptr<server_env> m_server_env;

    bool check_initialized() const {
        if (!m_server_env) {
            std::cerr << "Error: Server service not initialized" << std::endl;
            return false;
        }
        return true;
    }

    std::string construct_ssh_command() const {
        std::stringstream ssh_cmd;
        ssh_cmd << "ssh -p " << m_server_env->get_SSH_PORT() << " "
                << m_server_env->get_SSH_USER() << "@" << m_server_env->get_SSH_HOST() << " ";
        return ssh_cmd.str();
    }

    std::string get_service_dir() const {
        return m_server_env->get_DROPSHELL_DIR() + "/" + m_service_name;
    }

    std::string get_script_dir() const {
        return get_service_dir() + "/template";
    }

    std::string get_env_path() const {
        return get_service_dir() + "/" + m_service_name + ".env";
    }

    bool check_service_dir_exists(const std::string& ssh_cmd) const {
        std::string check_dir_cmd = ssh_cmd + "'test -d " + get_service_dir() + "'";
        if (system(check_dir_cmd.c_str()) != 0) {
            std::cerr << "Error: Service directory not found on server - has it been installed?" << std::endl;
            return false;
        }
        return true;
    }

    bool check_env_file_exists(const std::string& ssh_cmd) const {
        std::string check_env_cmd = ssh_cmd + "'test -f " + get_env_path() + "'";
        if (system(check_env_cmd.c_str()) != 0) {
            std::cerr << "Error: Service environment file not found on server" << std::endl;
            return false;
        }
        return true;
    }

    bool run_remote_command(const std::string& ssh_cmd, const std::string& command, const std::string& error_msg) const {
        if (system(command.c_str()) != 0) {
            std::cerr << "Error: " << error_msg << std::endl;
            return false;
        }
        return true;
    }

public:
    server_service() : m_server_name(""), m_service_name(""), m_server_env(nullptr) {}

    bool init(const std::string& server_name, const std::string& service_name) {
        std::string user_dir;
        if (!get_user_directory(user_dir)) {
            std::cerr << "Error: User directory not set" << std::endl;
            return false;
        }
        
        // Check if server exists
        fs::path server_dir = fs::path(user_dir) / "servers" / server_name;
        if (!fs::exists(server_dir)) {
            std::cerr << "Error: Server '" << server_name << "' not found" << std::endl;
            return false;
        }
        
        // Check if service env file exists
        fs::path service_env = server_dir / (service_name + ".env");
        if (!fs::exists(service_env)) {
            std::cerr << "Error: Service environment file not found: " << service_env.string() << std::endl;
            return false;
        }
        
        // Initialize server environment
        try {
            m_server_env = std::make_unique<server_env>(server_dir.string());
            if (!m_server_env->is_valid()) {
                std::cerr << "Error: Invalid server environment" << std::endl;
                return false;
            }
        } catch (const std::exception& e) {
            std::cerr << "Error: Failed to initialize server environment: " << e.what() << std::endl;
            return false;
        }
        
        m_server_name = server_name;
        m_service_name = service_name;
        return true;
    }

    bool install() {
        if (!check_initialized()) return false;
        
        // Check if template exists
        template_manager tm;
        template_info info;
        if (!tm.get_template_info(m_service_name, info)) {
            std::cerr << "Error: Template '" << m_service_name << "' not found" << std::endl;
            return false;
        }
        
        std::string ssh_cmd = construct_ssh_command();
        std::string service_dir = get_service_dir();
        
        // Create service directory
        std::string mkdir_cmd = ssh_cmd + "'mkdir -p " + service_dir + "/template'";
        if (!run_remote_command(ssh_cmd, mkdir_cmd, "Failed to create service directory")) {
            return false;
        }
        
        // Check if rsync is installed on remote host
        std::string check_rsync_cmd = ssh_cmd + "'which rsync > /dev/null 2>&1'";
        if (!run_remote_command(ssh_cmd, check_rsync_cmd, "rsync is not installed on the remote host")) {
            return false;
        }

        // Copy template files
        std::cout << "Copying template files from " << info.path << " to " << service_dir << "/template/" << std::endl;
        std::string rsync_cmd = "rsync --delete -zrpc -e 'ssh -p " + m_server_env->get_SSH_PORT() + "' " + 
                               info.path + "/ " + 
                               m_server_env->get_SSH_USER() + "@" + m_server_env->get_SSH_HOST() + ":" + 
                               service_dir + "/template/";
        if (!run_remote_command(ssh_cmd, rsync_cmd, "Failed to copy template files")) {
            return false;
        }
        
        // Copy service env file
        std::string user_dir;
        if (!get_user_directory(user_dir)) {
            std::cerr << "Error: User directory not set" << std::endl;
            return false;
        }
        fs::path service_env = fs::path(user_dir) / "servers" / m_server_name / (m_service_name + ".env");
        std::string scp_cmd = "scp -P " + m_server_env->get_SSH_PORT() + " " + 
                             service_env.string() + " " + 
                             m_server_env->get_SSH_USER() + "@" + m_server_env->get_SSH_HOST() + ":" + 
                             service_dir + "/" + m_service_name + ".env";
        if (!run_remote_command(ssh_cmd, scp_cmd, "Failed to copy service environment file")) {
            return false;
        }
        
        // Run install script
        std::string install_cmd = ssh_cmd + "'cd " + service_dir + "/template && /bin/bash _install.sh " + 
                                 service_dir + "/" + m_service_name + ".env'";
        if (!run_remote_command(ssh_cmd, install_cmd, "Failed to run install script")) {
            return false;
        }
        
        return true;
    }

    bool run_command(const std::string& command) {
        if (!check_initialized()) return false;
        
        std::string ssh_cmd = construct_ssh_command();
        std::string script_dir = get_script_dir();
        std::string script_path_and_command = script_dir + "/" + command + ".sh";
        
        if (!check_service_dir_exists(ssh_cmd)) return false;
        
        // Check if command script exists
        std::string check_script_cmd = ssh_cmd + "'test -f " + script_path_and_command + "'";
        if (!run_remote_command(ssh_cmd, check_script_cmd, "Command script '" + command + ".sh' not found")) {
            return false;
        }
        
        if (!check_env_file_exists(ssh_cmd)) return false;
        
        // Run the command
        std::string run_cmd = ssh_cmd + "'cd " + script_dir + 
                            " && /bin/bash " + script_path_and_command + " " + get_env_path() + "'";
        if (!run_remote_command(ssh_cmd, run_cmd, "Command returned error code: " + script_path_and_command)) {
            return false;
        }
        
        return true;
    }

    bool backup() {
        if (!check_initialized()) return false;
        
        std::string ssh_cmd = construct_ssh_command();
        std::string script_dir = get_script_dir();
        std::string script_path = script_dir + "/_backup.sh";
        
        if (!check_service_dir_exists(ssh_cmd)) return false;
        
        // Check if backup script exists
        std::string check_script_cmd = ssh_cmd + "'test -f " + script_path + "'";
        if (!run_remote_command(ssh_cmd, check_script_cmd, "Backup script not found")) {
            return false;
        }
        
        if (!check_env_file_exists(ssh_cmd)) return false;

        // Create backups directory on server if it doesn't exist
        std::string server_backups_dir = m_server_env->get_DROPSHELL_DIR() + "/backups";
        std::string mkdir_cmd = ssh_cmd + "'mkdir -p " + server_backups_dir + "'";
        if (!run_remote_command(ssh_cmd, mkdir_cmd, "Failed to create backups directory on server")) {
            return false;
        }

        // Create backups directory locally if it doesn't exist
        std::string user_dir;
        if (!get_user_directory(user_dir)) {
            std::cerr << "Error: User directory not set" << std::endl;
            return false;
        }
        fs::path local_backups_dir = fs::path(user_dir) / "backups";
        if (!fs::exists(local_backups_dir)) {
            fs::create_directories(local_backups_dir);
        }

        // Get current datetime for backup filename
        auto now = std::chrono::system_clock::now();
        auto time = std::chrono::system_clock::to_time_t(now);
        std::stringstream datetime;
        datetime << std::put_time(std::localtime(&time), "%Y-%m-%d_%H-%M-%S");
        
        // Construct backup filename
        std::string backup_filename = m_server_name + "-" + m_service_name + "-" + datetime.str() + ".tgz";
        std::string server_backup_path = server_backups_dir + "/" + backup_filename;
        std::string local_backup_path = (fs::path(user_dir) / "backups" / backup_filename).string();

        // Run backup script
        std::string backup_cmd = ssh_cmd + "'cd " + script_dir + 
                                " && /bin/bash backup.sh " + get_env_path() + " " + server_backup_path + "'";
        if (!run_remote_command(ssh_cmd, backup_cmd, "Backup script failed")) {
            return false;
        }

        // Copy backup file from server to local
        std::string scp_cmd = "scp -P " + m_server_env->get_SSH_PORT() + " " + 
                             m_server_env->get_SSH_USER() + "@" + m_server_env->get_SSH_HOST() + ":" + 
                             server_backup_path + " " + local_backup_path;
        if (system(scp_cmd.c_str()) != 0) {
            std::cerr << "Error: Failed to copy backup file from server" << std::endl;
            return false;
        }

        std::cout << "Backup created successfully: " << local_backup_path << std::endl;
        return true;
    }
};

} // namespace dropshell 